@@ -20,7 +20,8 @@ import Options.Applicative hiding (many)
 import qualified Options.Applicative as O
 import Options.Applicative.Types (readerAsk)
 
-import Development.Shake hiding (withTempDir)
+import Development.Shake.Fancy hiding (withTempDir)
+import qualified Development.Shake.Fancy as S
 import Development.Shake.Classes
 import Development.Shake.FilePath
 
@@ -174,10 +175,10 @@ splitDebName filename
 ensureVersion :: String -> String -> Action ()
 ensureVersion s v = do
     ex <- doesFileExist $ "p" </> s </> "debian" </> "changelog"
-    unless ex $ do
+    unless ex $
         fail $ "I do not know about package " ++ s
     v' <- versionOfSource s
-    when (v /= v') $ do
+    when (v /= v') $
         fail $ "Cannot build " ++ s ++ " version " ++ v ++ ", as we have " ++ v' ++ "."
 
 ensureArch :: String -> Action ()
@@ -234,7 +235,7 @@ parseFlatRel = flatRels . parseRels
 
     parseRels :: String -> Relations
     parseRels s = case parseRelations s of
-      Left pe ->  error $ "Failed to parse relations " ++ (show pe)
+      Left pe ->  error $ "Failed to parse relations " ++ show pe
       Right rel -> rel
 
 fixupScript :: [String] -> String
@@ -270,7 +271,9 @@ newtype GetExcludedSources = GetExcludedSources () deriving (Show,Typeable,Eq,Ha
 newtype GetDebBuiltBy = GetDebBuiltBy String  deriving (Show,Typeable,Eq,Hashable,Binary,NFData)
 newtype GetBinToDeb = GetBinToDeb String  deriving (Show,Typeable,Eq,Hashable,Binary,NFData)
 
-newtype GetArch = GetArch () deriving (Show,Typeable,Eq,Hashable,Binary,NFData)
+newtype GetArch = GetArch () deriving (Typeable,Eq,Hashable,Binary,NFData)
+
+instance Show GetArch where show (GetArch ()) = "querying architecture"
 
 -- Find dependencies on binary packages we build ourselves.
 -- This is needed, because those binary packages don't need to be
@@ -279,23 +282,26 @@ newtype GetArch = GetArch () deriving (Show,Typeable,Eq,Hashable,Binary,NFData)
 -- Finding library packages isn't necessary, because they were already
 -- built as a build-dependency.
 extendBuildDepends :: FilePath -> (String -> Action (Maybe String)) -> [String] -> [String] -> Action [String]
-extendBuildDepends targetDir binToDeb ourBins buildDeps = (buildDeps ++) <$> concat <$> mapM ourDepends buildDeps
-    where ourDepends dep = if dep `notElem` ourBins
-                           then do
-                             let debFile = targetDir </> dep
-                             need [debFile]
-                             Stdout depList <- cmd (Traced "")
-                                                   [ "dpkg-deb"
-                                                   , "--field"
-                                                   , debFile
-                                                   , "Depends"
-                                                   ]
-                             let depPkgs = map cleanDep $ splitOn "," depList
-                             depends <- catMaybes <$> mapM binToDeb depPkgs
-                             recursive <- extendBuildDepends targetDir binToDeb ourBins depends
-                             return $ depends ++ recursive
-                           else return []
-          cleanDep = takeWhile (not . isSpace) . dropWhile isSpace
+extendBuildDepends targetDir binToDeb ourBins buildDeps = do
+    let notOurBin = map (targetDir </>) $ filter (`notElem` ourBins) buildDeps
+    -- First need all
+    need notOurBin
+    -- The look for additional dependencies
+    (buildDeps ++) <$> concat <$> mapM ourDepends notOurBin
+  where
+    ourDepends debFile = do
+        Stdout depList <- cmdWrap "dpkg-deb --field Depends" $ cmd
+                              [ "dpkg-deb"
+                              , "--field"
+                              , debFile
+                              , "Depends"
+                              ]
+        let depPkgs = map cleanDep $ splitOn "," depList
+        depends <- catMaybes <$> mapM binToDeb depPkgs
+        recursive <- extendBuildDepends targetDir binToDeb ourBins depends
+        return $ depends ++ recursive
+
+    cleanDep = takeWhile (not . isSpace) . dropWhile isSpace
 
 manpage :: String
 manpage = unlines [ "TODO" ]
@@ -307,7 +313,7 @@ main = do
             let failure = parserFailure (prefs idm) opts ShowHelpText mempty
             let message = renderFailure failure "dht make-all"
             putStrLn . unlines . drop 2 . lines . fst $ message
-        _ -> do execParser opts >>= run
+        _ -> execParser opts >>= run
   where
     opts = info (helper <*> confSpec)
         ( fullDesc
@@ -358,7 +364,7 @@ shakeMain conf@(Conf {..}) sources = do
     if null targets then want ["all"] else want targets
 
     getArch' <- addOracle $ \GetArch{} -> do
-        Stdout archString <- cmd
+        Stdout archString <- cmdWrap "schroot" $ cmd
             [ "schroot"
             , "-d", "/"
             , "-c", schrootName
@@ -390,7 +396,7 @@ shakeMain conf@(Conf {..}) sources = do
             | [deb,source] <- words <$> builtBy
             ]
 
-    getDebBuiltBy <- addOracle $ \(GetDebBuiltBy bin) -> do
+    getDebBuiltBy <- addQuietOracle $ \(GetDebBuiltBy bin) -> do
         map <- debBuiltByMap ()
         return $ M.lookup bin map
 
@@ -405,7 +411,7 @@ shakeMain conf@(Conf {..}) sources = do
             , let (pkgname,_,_) = splitDebName deb
             ]
 
-    getBinToDeb <- addOracle $ \(GetBinToDeb bin) -> M.lookup bin <$> binToDebMap ()
+    getBinToDeb <- addQuietOracle $ \(GetBinToDeb bin) -> M.lookup bin <$> binToDebMap ()
 
     let binToDeb :: String -> Action (Maybe String)
         binToDeb = getBinToDeb . GetBinToDeb
@@ -419,7 +425,7 @@ shakeMain conf@(Conf {..}) sources = do
         writeFileChanged out $ unlines $ map (flip (uncurry changesFileName) arch) versioned
 
     targetDir </> "cache/binaries/*.txt" %> \out -> do
-        let s = dropExtension $ takeFileName $ out
+        let s = dropExtension $ takeFileName out
         putNormal $ "# enumerating binaries of " ++ show s
         arch <- getArch
         pkgs <- binaryPackagesOfSource s arch
@@ -436,7 +442,7 @@ shakeMain conf@(Conf {..}) sources = do
         versioned <- forM sources $ \s -> do
             v <- versionOfSource s
             return (s,v)
-        need $ [ targetDir </> sourceFileName s v | (s,v) <- versioned ]
+        need [ targetDir </> sourceFileName s v | (s,v) <- versioned ]
 
     -- Binary packages depend on the corresponding changes file log
     targetDir </> "*.deb" %> \out -> do
@@ -511,7 +517,10 @@ shakeMain conf@(Conf {..}) sources = do
             let repoDir = tmpdir </> "repo"
             liftIO $ createDirectory repoDir
             forM_ localDebs $ \p -> liftIO $ linkOrCopyFile ("lab" </> p) (repoDir </> p)
-            unit $ cmd (Cwd repoDir)  (EchoStderr False) (FileStdout (repoDir </> "Packages"))
+            unit $ cmdWrap "dpkg-scanpackages" $ cmd
+                (Cwd repoDir)
+                (EchoStderr False)
+                (FileStdout (repoDir </> "Packages"))
                 ["dpkg-scanpackages", "."]
 
             -- Always pass the distribution in the changelog to sbuild, otherwise
@@ -522,7 +531,9 @@ shakeMain conf@(Conf {..}) sources = do
             dist <- distributionOfSource source
 
             -- Run sbuild
-            Exit c <- cmd (Cwd targetDir) (EchoStdout False) $
+            Exit c <- cmdWrap "sbuild" $ cmd
+                (Cwd targetDir)
+                (EchoStdout False) $
                 ["sbuild"
                 , "-c", schrootName
                 , "-A"
@@ -532,12 +543,13 @@ shakeMain conf@(Conf {..}) sources = do
                 , "--extra-repository=deb [trusted=yes] file://" ++ repoDir ++ " ./"
                 , dsc
                 ] ++ sbuildArgs
-            if (c == ExitSuccess)
-            then do
+            if c == ExitSuccess
+            then
                 -- Add the sources to the changes file. We do not simply pass
                 -- "-s" to sbuild, as it would make sbuild re-create and override the .dsc file
                 -- which confuses the build system.
-                unit $ cmd ["changestool", targetDir </> changes, "adddsc", targetDir </> dsc]
+                unit $ cmdWrap "changestool" $
+                    cmd ["changestool", targetDir </> changes, "adddsc", targetDir </> dsc]
             else do
                 putNormal $ "Failed to build " ++ source ++ "_" ++ version
                 ex <- liftIO $ System.Directory.doesFileExist out
@@ -551,7 +563,8 @@ shakeMain conf@(Conf {..}) sources = do
         ensureVersion source version
         sourceFiles <- getDirectoryFiles ("p" </> source) ["debian//*"]
         need [ "p" </> source </> f | f <- sourceFiles]
-        unit $ cmd  (EchoStdout False) (Traced "debian2dsc") "dht" "debian2dsc" "-o" targetDir ("p" </> source </> "debian")
+        unit $ cmdWrap "debian2dsc" $ cmd (EchoStdout False)
+                "dht" "debian2dsc" "-o" targetDir ("p" </> source </> "debian")
 
     phonys $ \source -> if source `elem` sources
                         then Just $ do
